package com.qmth.distributed.print.business.service.impl;

import com.alibaba.excel.EasyExcel;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.boot.api.exception.ApiException;
import com.qmth.distributed.print.business.entity.TCFinalScore;
import com.qmth.distributed.print.business.mapper.TCFinalScoreMapper;
import com.qmth.distributed.print.business.service.PrintCommonService;
import com.qmth.distributed.print.business.service.TCFinalScoreService;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.FieldUniqueEnum;
import com.qmth.teachcloud.common.util.JacksonUtil;
import com.qmth.teachcloud.common.util.ResultUtil;
import com.qmth.teachcloud.common.util.ServletUtil;
import com.qmth.teachcloud.mark.entity.MarkPaper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.*;

/**
 * <p>
 * 期末考试成绩表 服务实现类
 * </p>
 *
 * @author wangliang
 * @since 2024-02-18
 */
@Service
public class TCFinalScoreServiceImpl extends ServiceImpl<TCFinalScoreMapper, TCFinalScore> implements TCFinalScoreService {
    private final static Logger log = LoggerFactory.getLogger(TCFinalScoreServiceImpl.class);

    @Resource
    PrintCommonService printCommonService;

    @Resource
    TCFinalScoreService tcFinalScoreService;

    /**
     * 导入期末成绩excel
     *
     * @param file
     * @param examId
     * @param courseCode
     * @param paperNumber
     * @return
     * @throws IOException
     */
    @Override
    @Transactional
    public Map<String, String> finalScoreExcelImport(MultipartFile file, Long examId, String courseCode, String paperNumber) throws IOException {
        log.debug("导入Excel开始...");
        long start = System.currentTimeMillis();
        MarkPaper markPaper = printCommonService.scoreImportExcelVaild(file, examId, paperNumber);

        Map<String, String> messageMap = new LinkedHashMap<>();
        try {
            StringJoiner errorData = new StringJoiner("");
            StringJoiner successData = new StringJoiner("");
            List<Map<String, String>> list = EasyExcel.read(file.getInputStream()).headRowNumber(1).sheet(0).doReadSync();
            log.info("list:{}", JacksonUtil.parseJson(list));

            SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
            List<TCFinalScore> tcFinalScoreList = new ArrayList<>(list.size());
            Map<String, String> headMap = list.get(0);
            for (Map.Entry<String, String> entry : headMap.entrySet()) {
                if (Objects.equals(entry.getKey(), 0) && Objects.nonNull(entry.getValue()) &&
                        !Objects.equals(entry.getValue().trim(), "学号")) {
                    throw ExceptionResultEnum.ERROR.exception("excel表头第一行为学号");
                } else if (Objects.equals(entry.getKey(), 1) && Objects.nonNull(entry.getValue()) &&
                        !Objects.equals(entry.getValue().trim(), "姓名")) {
                    throw ExceptionResultEnum.ERROR.exception("excel表头第二行为姓名");
                } else if (Objects.equals(entry.getKey(), 2) && Objects.nonNull(entry.getValue()) &&
                        !Objects.equals(entry.getValue().trim(), "成绩")) {
                    throw ExceptionResultEnum.ERROR.exception("excel表头第三行为成绩");
                }
            }

            for (int i = 1; i < list.size(); i++) {
                Map<String, String> objectMap = list.get(i);
                boolean error = false;
                JSONArray jsonArray = new JSONArray();
                TCFinalScore tcFinalScore = new TCFinalScore(examId, courseCode, markPaper.getCourseName(), paperNumber, sysUser.getId());
                for (Map.Entry<String, String> entry : objectMap.entrySet()) {
                    JSONObject jsonObject = new JSONObject();
                    if (Objects.isNull(entry.getValue()) || Objects.equals(entry.getValue().trim(), "")) {
                        errorData.add("excel第" + i + "行[").add(headMap.get(entry.getKey()) + "]为空;").add("\r\n");
                        error = true;
                    } else {
                        String head = headMap.get(entry.getKey());
                        Objects.requireNonNull(head, "表头信息异常");
                        if (Objects.equals(head.trim(), "学号")) {
                            tcFinalScore.setStudentCode(entry.getValue());
                        } else if (Objects.equals(head.trim(), "姓名")) {
                            tcFinalScore.setName(entry.getValue());
                        } else if (Objects.equals(head.trim(), "成绩")) {
                            tcFinalScore.setScore(Double.valueOf(entry.getValue()));
                        } else {
                            jsonObject.put("name", headMap.get(entry.getKey()));
                            jsonObject.put("score", entry.getValue());
                            jsonArray.add(jsonObject);
                        }
                    }
                }
                if (jsonArray.size() > 0) {
                    tcFinalScore.setScoreDetail(jsonArray.toJSONString());
                }
                if (!error) {
                    tcFinalScoreList.add(tcFinalScore);
                }
            }
            if (!CollectionUtils.isEmpty(tcFinalScoreList)) {
                successData.add("共导入" + tcFinalScoreList.size() + "条数据");

                QueryWrapper<TCFinalScore> tcScoreEndExamQueryWrapper = new QueryWrapper<>();
                tcScoreEndExamQueryWrapper.lambda().eq(TCFinalScore::getExamId, examId)
                        .eq(TCFinalScore::getCourseCode, courseCode)
                        .eq(TCFinalScore::getCourseName, markPaper.getCourseName())
                        .eq(TCFinalScore::getPaperNumber, paperNumber);
                tcFinalScoreService.remove(tcScoreEndExamQueryWrapper);
                tcFinalScoreService.saveBatch(tcFinalScoreList);
            }
            messageMap.put("正确信息", successData.length() > 0 ? successData.toString() : "无");
            messageMap.put("错误信息", errorData.length() > 0 ? errorData.toString() : "无");
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
            if (e instanceof DuplicateKeyException) {
                String errorColumn = e.getCause().toString();
                String columnStr = errorColumn.substring(errorColumn.lastIndexOf("key") + 3, errorColumn.length()).replaceAll("'", "");
                throw ExceptionResultEnum.SQL_ERROR.exception("[" + FieldUniqueEnum.convertToTitle(columnStr) + "]数据不允许重复插入");
            } else if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, ((ApiException) e).getCode(), e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        }
        log.debug("导入Excel结束...");
        long end = System.currentTimeMillis();
        log.info("============耗时{}秒============:", (end - start) / 1000);
        return messageMap;
    }

    /**
     * 期末成绩列表（分页）
     *
     * @param iPage
     * @param examId
     * @param courseCode
     * @param paperNumber
     * @return
     */
    @Override
    public IPage<TCFinalScore> finalScoreList(IPage<Map> iPage, Long examId, String courseCode, String paperNumber) {
        return this.baseMapper.finalScoreList(iPage, examId, courseCode, paperNumber);
    }
}
